package org.sakaiproject.syllabi.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Collections;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.RequestDispatcher;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.sakaiproject.api.app.syllabus.GatewaySyllabus;
import org.sakaiproject.api.app.syllabus.SyllabusAttachment;
import org.sakaiproject.component.cover.ComponentManager;
import org.sakaiproject.component.cover.ServerConfigurationService;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.site.api.Site;
import org.sakaiproject.site.cover.SiteService;
import org.sakaiproject.coursemanagement.api.Section;
import org.sakaiproject.authz.api.AuthzGroupService;
import org.sakaiproject.coursemanagement.api.CourseManagementService;

public class SyllabiDisplayServlet extends HttpServlet {

	private Log logger = LogFactory.getLog(SyllabiDisplayServlet.class);

	public void init(ServletConfig config) throws ServletException 
	{
		super.init(config);
		String serverUrl = ServerConfigurationService.getString("syllabi.displayHost");
		if ((serverUrl == null) || "".equals(serverUrl)) {
			serverUrl = ServerConfigurationService.getServerUrl();
		}

		String relativeUrl = config.getInitParameter("authUrl");
		authUrl = serverUrl + relativeUrl;
		display = config.getInitParameter("display");
		if (! "auth".equals(display))
		{
			display = "public";
		}
		rostersStyle = ServerConfigurationService.getString("syllabi.rostersStyle");
	}

        public void setLogger(Log logger) {
                this.logger = logger;
        }

        protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException
        {
		String TOKEN = "restricted";
		String BEFORE = "/"+TOKEN+"$";
		String AFTER = "#"+TOKEN;
		String[] pathinfoParts = req.getPathInfo().split("/");
		if (logger.isDebugEnabled()) {
			logger.debug("pathinfoParts n=="+pathinfoParts.length);
			for (int i = 0; i < pathinfoParts.length; i++) {
				logger.debug("pathinfoParts["+i+"]=="+pathinfoParts[i]);
			}
		}
		// pathinfoParts[0] == null and precedes leading /
		String siteId = null;
		if (pathinfoParts.length >= 2) {
			siteId = pathinfoParts[1];
		}
		if (logger.isDebugEnabled()) logger.debug("siteId=="+siteId);
		String anchor = null;
		if (pathinfoParts.length >= 3) {
			anchor = pathinfoParts[2];
		}
		if (logger.isDebugEnabled()) logger.debug("anchor=="+anchor);
		// pathinfoParts[3] == null and follows trailing /
		if (TOKEN.equals(anchor)) {
			String oldpath = req.getRequestURL().toString();
			if (logger.isDebugEnabled()) logger.debug("oldpath=="+oldpath);
			String newpath = oldpath.replaceFirst(BEFORE, AFTER);
			if (logger.isDebugEnabled()) logger.debug("newpath=="+newpath);
			res.setContentType("text/plain");
			PrintWriter out= null;
			try 
			{
				out= res.getWriter();
			}
			catch (IOException ioe)
			{
				throw new ServletException(ioe);
			}
			res.setStatus(res.SC_MOVED_TEMPORARILY);
			res.setHeader("Location", newpath);
		}

		//String siteId = req.getPathInfo().substring(1); ???
                Site site = null;
                try
                {
                        site = SiteService.getSite(siteId);
			
                        List netbadgedSyllabusList = null;
			int netbadgedN = 0;
			try
			{
                	        Object syllabusService = ComponentManager.get("org.sakaiproject.api.app.syllabus.SyllabusService");
                               	Class syllabusServiceClass = syllabusService.getClass();
                       	 	Class[] paramsClasses = new Class[2]; //2
                               	paramsClasses[0] = java.lang.String.class;
                       		paramsClasses[1] = java.lang.String.class;
                               	Method getMessagesByView = syllabusServiceClass.getMethod("getMessagesByView", paramsClasses);
                               	String paramSiteId = site.getId();
                               	netbadgedSyllabusList = (ArrayList) getMessagesByView.invoke(syllabusService, new Object[] { paramSiteId, "netbadged" }); 
                               	//netbadgedSyllabusList = getMessagesByView(paramSiteId, "netbadged"); //TRY IT HERE 
                       	} catch (Exception reflectionEx)
                       	{
				logger.error(reflectionEx.getMessage());
				if (null != reflectionEx.getCause())
				{
					logger.error(reflectionEx.getCause().getMessage());
				}
                       	}
			if (netbadgedSyllabusList != null)
			{
				netbadgedN = netbadgedSyllabusList.size();
			}

                        List publicSyllabusList = null;
			int publicN = 0;
                        // get the public syllabus
                        try
                        {
                                Object syllabusService = ComponentManager.get("org.sakaiproject.api.app.syllabus.SyllabusService");
                                Class syllabusServiceClass = syllabusService.getClass();
                                Class[] paramsClasses = new Class[1];
                                paramsClasses[0] = java.lang.String.class;
                                Method getMessages = syllabusServiceClass.getMethod("getMessages", paramsClasses);
                                String paramSiteId = site.getId();
                                publicSyllabusList = (ArrayList) getMessages.invoke(syllabusService, new Object[] { paramSiteId });
                        }
                        catch (Exception reflectionEx)
                        {
                        }
			if (publicSyllabusList != null) {
				publicN = publicSyllabusList.size();
			}

			if (logger.isDebugEnabled()) logger.debug("netbadgedN=="+netbadgedN);
			RequestDispatcher disp;
			disp = getServletContext(  ).getRequestDispatcher("/syllabi/view.jsp");
			req.setAttribute("pubSyls", publicSyllabusList);
			req.setAttribute("siteTitle", site.getTitle());
			req.setAttribute("siteId", siteId);
			req.setAttribute("rostersStyle", rostersStyle);
			List NULL_LIST = new ArrayList();
			if ("public".equals(display))
			{
	                	if (netbadgedN > 0)
				{
					req.setAttribute("authUrl", authUrl);
				}
				netbadgedSyllabusList = NULL_LIST;
			}
			req.setAttribute("authSyls", netbadgedSyllabusList);
			CourseManagementService cms = (CourseManagementService) ComponentManager.get(CourseManagementService.class);
			AuthzGroupService authzGroupService = (AuthzGroupService) ComponentManager.get(AuthzGroupService.class);
			Set set = authzGroupService.getProviderIds("/site/"+siteId);
			List rosterList = new ArrayList(set.size());
			Iterator iter = set.iterator();
			while (iter.hasNext()) {
				String sectionEid = (String) iter.next();
				Section section = cms.getSection(sectionEid);
				String sectionTitle = section.getTitle();
				rosterList.add(sectionTitle);
			}
			Collections.sort(rosterList);
			req.setAttribute("rosters", rosterList);
			disp.forward(req, res);
                }
		catch (IdUnusedException err)
                {
			logger.error("error thrown");
                }
		catch (IOException ioe) 
		{
			logger.error("error thrown");
		}
		finally
		{
		}
	}

        protected void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException
        {
		doGet(req, res);
        }

	private String display = null;
	private String authUrl = null;
	private String rostersStyle = null;


}
