package org.sakaiproject.syllabi.servlets;

import java.io.PrintWriter;
import java.io.BufferedReader; 
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.sakaiproject.component.cover.ServerConfigurationService;
import org.sakaiproject.component.cover.ServerConfigurationService;

public class SyllabiDuplicatesServlet extends HttpServlet
{

        private Log logger = LogFactory.getLog(SyllabiListServlet.class);

	private static final String SEPARATOR = "\t";

	private String username = ServerConfigurationService.getString("username@javax.sql.BaseDataSource", "testuser");
	private String password = ServerConfigurationService.getString("password@javax.sql.BaseDataSource", "testpass");
	private String dbUrl = ServerConfigurationService.getString("url@javax.sql.BaseDataSource", "");
	private String dbDriver = ServerConfigurationService.getString("driverClassName@javax.sql.BaseDataSource", "");
        private String query = null;

        public void init(ServletConfig config) throws ServletException
        {
                super.init(config);
                query = config.getInitParameter("query");
        }

	public void setLogger(Log logger) {
		this.logger = logger;
	}

        protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException
        {
        	Connection conn = null;
		Statement s = null;
		ResultSet rs = null;
		PrintWriter pw = null;

        	try
        	{
       		 	Class.forName (dbDriver).newInstance ();
       			conn = DriverManager.getConnection (dbUrl, username, password);
			s = conn.createStatement ();
			s.executeQuery (query);
			rs = s.getResultSet ();
			res.setContentType("text/html");
			pw = res.getWriter();

			String label = "Rosters usage preventing SIS syllabus links";
			String title = "<title>" + label + "</title>";
			String header = "<h3>" + label + "</h3>";
			String line = "<html><head>" + title + "</head><body>" + header + "<table>";
			pw.println(line);

			line = "<TH style=\"border-style:solid;border-width:1px\">Roster</TH>";
			line += "<TH style=\"border-style:solid;border-width:1px\">Site ID</TH>";
			line += "<TH style=\"border-style:solid;border-width:1px\">Site Title</TH>";
			line += "<TH style=\"border-style:solid;border-width:1px\">Syllabus view</TH>";
			line = "<TR>" + line + "</TR>";
			pw.println(line);

			while (rs.next ())
			{
				String rosterId = rs.getString ("roster");
				String siteId = rs.getString ("site_id");
				String siteTitle = rs.getString ("site_title");
				String syllabusType = rs.getString ("syllabus_type");
				line = "<TD style=\"border-style:solid;border-width:1px\">" + rosterId + "</TD>";
				line += "<TD style=\"border-style:solid;border-width:1px\">" + siteId + "</TD>";
				line += "<TD style=\"border-style:solid;border-width:1px\">" + siteTitle + "</TD>";
				line += "<TD style=\"border-style:solid;border-width:1px\">" + syllabusType + "</TD>";
				line = "<TR>" + line + "</TR>";
				pw.println(line);
   			}

			pw.println("</table>");

			line = "<p>World Readable:  viewable by anyone browsing the Schedule of Classes in SIS</p>";
			pw.println(line);

			line = "<p>UVa Only:  NetBadge authentication required to access from SIS</p>";
			pw.println(line);

			pw.println("<div style=\"visibility:hidden\">testing</div></body><html>");
		}
           	catch (Exception e)
           	{
			logger.error("WDN SYLLABI SERVLET problem");
			logger.error(e.getMessage());
			if (e.getCause() != null) {
				logger.error(e.getCause().getMessage());
			}
           	}
		finally
           	{
			if (pw != null)
			{
				try
				{
					pw.close ();
				}
				catch (Exception e)
				{
					logger.error("SYLLABI LIST SERVLET pw close error");
				}
			}
			if (conn != null)
			{
				try
				{
					conn.close ();
					logger.error("WDN SYLLABI SERVLET Database connection terminated");
				}
				catch (Exception e)
				{
					logger.error("SYLLABI LIST SERVLET conn close error");
				}
			}
			if (s != null)
			{
				try
				{
					s.close ();
					logger.error("WDN SYLLABI SERVLET Database connection terminated");
				}
				catch (Exception e)
				{
					logger.error("SYLLABI LIST SERVLET statement close error");
				}
			}
			if (rs != null)
			{
				try
				{
					rs.close ();
					logger.error("WDN SYLLABI SERVLET Database connection terminated");
				}
				catch (Exception e)
				{
					logger.error("SYLLABI LIST SERVLET result set close error");
				}
			}
           	}
	}

	protected  void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		doGet(req, resp);
	}

} // SyllabiDuplicatesServlet

