package org.sakaiproject.syllabi.servlets;

import java.io.PrintWriter;
import java.io.BufferedReader; 
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.sakaiproject.component.cover.ServerConfigurationService;
import org.sakaiproject.component.cover.ServerConfigurationService;

public class SyllabiListServlet extends HttpServlet
{

        private Log logger = LogFactory.getLog(SyllabiListServlet.class);

	private static final String SEPARATOR = "\t";

	private String username = ServerConfigurationService.getString("username@javax.sql.BaseDataSource", "testuser");
	private String password = ServerConfigurationService.getString("password@javax.sql.BaseDataSource", "testpass");
	private String dbUrl = ServerConfigurationService.getString("url@javax.sql.BaseDataSource", "");
	private String dbDriver = ServerConfigurationService.getString("driverClassName@javax.sql.BaseDataSource", "");
        private String publicUrl = null;
        private Pattern[] patternsToIgnore = {};
        private String query = null;
        private String[] stmts = null;
	private int nSelects = 0;
	private int iSelect = -1;

        public void init(ServletConfig config) throws ServletException
        {
		// too hard to set debug logging before this init, hence logger.info() used here
                super.init(config);
                String serverUrl = ServerConfigurationService.getString("syllabi.displayHost");
                if ((serverUrl == null) || "".equals(serverUrl)) {
                        serverUrl = ServerConfigurationService.getServerUrl();
                }

                String relativeUrl = config.getInitParameter("publicUrl");
                publicUrl = serverUrl + relativeUrl;
                query = config.getInitParameter("query");

		/* UVa SAK-2865: for separate execution of added temp table and index creation and cleanup */
		logger.debug("SYLLABI LIST SERVLET init: query = " + query);
		stmts = query.split(";\n"); // tried ";$" but it lit didn't cut it
		logger.info("SYLLABI LIST SERVLET init: stmts.length = " + stmts.length);
		if (stmts.length == 0) {
			logger.info("SYLLABI LIST SERVLET init: using single-stmt query");
			stmts = new String[1];
			stmts[0] = query;
			iSelect = 0;
			nSelects = 1;
		} else {
			for (int i=0; i<stmts.length; i++) {
				logger.debug("SYLLABI LIST SERVLET init: another stmt, at [" + i + "], = " + stmts[i]);
				String temp = stmts[i].replaceAll("/\\*.+\\*/",""); // so can't embed comment in comment . . .
				temp = temp.trim(); // includes line breaks
				if(temp.startsWith("select")) {
					iSelect = i;
					nSelects++;
					logger.info("SYLLABI LIST SERVLET init: another select, at [" + iSelect + "], nSelects now = " + nSelects);
				}
			}
			logger.info("SYLLABI LIST SERVLET init: nSelects = " + nSelects);
			if (nSelects != 1) {
				logger.fatal("SYLLABI LIST SERVLET init error: must have (only) 1 select stmt returning results");
				logger.fatal("SYLLABI LIST SERVLET init error: subsequent HTTP GETS WILL FAIL, SC=500");
			}
		}

		String temp = config.getInitParameter("regexsToIgnore");
		String[] regexsToIgnore = {};
		if (temp != null) {
			regexsToIgnore = temp.split("\n");
		}
		patternsToIgnore = new Pattern[regexsToIgnore.length];
		for (int i = 0; i < regexsToIgnore.length; i++) {
			patternsToIgnore[i] = Pattern.compile(regexsToIgnore[i]);
		}
        }

	public void setLogger(Log logger) {
		this.logger = logger;
	}

	public static final Vector arrayToVector (Object[] objects)
	{
		Vector temp = new Vector(objects.length);
		for (int i = 0; i < objects.length; i++) {
			temp.add(objects[i]);
		}
		return temp;
	}

	public final boolean ignore (String string)
	{
		boolean ignore = false;
		for (int i = 0; i < patternsToIgnore.length; i++) {
			Matcher m = patternsToIgnore[i].matcher(string);
			if (m.matches()) {
				ignore = true;
				break;
			}
		}
		return ignore;
	}


	public final String getKey(String roster)
	{

        	String[] partsTemp = roster.split("_");
		Vector parts = arrayToVector(partsTemp);

        	//   2008 3 AAS 405B-1 CLAS 304YF ...
        	//or 2008 3 C S 405B-1 CLAS 304YF ...
        	//or 1088 CHEM 181-1 CGAS 10919 ...

        	String term = (String) parts.remove(0);
        	if (! term.matches("^\\d\\d\\d\\d$")) {
			logger.error("bad term=="+term);
                	return null;
        	}

        	boolean oldschool = false;
        	String semester = "";
        	String dept = (String) parts.remove(0);
        	if (dept.matches("^\\d$")) {
                	semester = dept;
                	dept = (String) parts.remove(0);
                	//FIXUP TERM
			term = Integer.toString(Integer.parseInt(term) - 1900);
                	if ("2".equals(semester)) {
                        	term = term + "6";
                	} else if ("1".equals(semester)) {
                        	term = term + "2";
                	} else if ("3".equals(semester)) {
                        	term = term + "8";
                	} else {
				logger.error("bad semester=="+semester);
                        	return null;
                	}
                	oldschool = true;
        	}

        	String course = "";
        	String section = "";
        	String coursesection = (String) parts.remove(0);
        	if (! coursesection.matches("^\\d.+$")) {
                	dept = dept + coursesection;
                	if ("CS".equals(dept)) {
                	} else if ("CE".equals(dept)) {
                	} else if ("CHE".equals(dept)) {
                	} else if ("AM".equals(dept)) {
                	} else if ("EP".equals(dept)) {
                	} else if ("NE".equals(dept)) {
                	} else if ("ARH".equals(dept)) {
                	} else if ("LAR".equals(dept)) {
                	} else {
				logger.error("bad space dept=="+dept);
                        	return null;
                	}
                	coursesection = (String) parts.remove(0);
        	}
		String[] pp = coursesection.split("-");
		if (pp.length != 2) {
			logger.error("unexpected split on hyphen");
                	return null;
		}
        	course = pp[0];
        	section = pp[1];

        	if (oldschool) {
                	String school = (String) parts.remove(0);
                	String schedulenum = (String) parts.remove(0);
                	section = schedulenum.substring(1);
        	}

        	//   AAS 405B-1 CLAS 304YF ...
        	//or C S 405B-1 CLAS 304YF ...
        	//or 1088 CHEM 181-01RA CGAS 10919 ...

        	//may need to revisit how to handle iitd courses after seeing eventual ssp cm feed

		String key = term + SEPARATOR + dept + SEPARATOR + course + SEPARATOR + section;
		return key;
	}

	private static final void processBuffer(PrintWriter pw, String mode, Vector buffer)
	{
		if (
		   ("duplicate".equals(mode) && (buffer.size() > 1)) 
		|| ("singleton".equals(mode) && (buffer.size() == 1))
		   ) {
			while (! buffer.isEmpty()) {
				String temp = (String) buffer.remove(0);
       				pw.println(temp);
			}
		} else {
			buffer.clear();
		}
	}

        protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException
        {
		if (nSelects != 1) {
			res.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
			logger.fatal("WDN SYLLABI SERVLET doGet(): nSelects != 0");
			return;
		}

        	Connection conn = null;
		Statement s = null;
		ResultSet rs = null;
		PrintWriter pw = null;

        	try
        	{
       		 	Class.forName (dbDriver).newInstance ();
       			conn = DriverManager.getConnection (dbUrl, username, password);

			s = conn.createStatement ();

			for (int i=0; i<stmts.length; i++) {
				if (iSelect != i) { // accommodates temp tbl and index creation and cleanup
					s.executeUpdate (stmts[i]);
					continue;
				}

				// continue to process result-returning select
				int nOutputLines = 0;
				s.executeQuery (stmts[i]);
				rs = s.getResultSet ();
				int count = 0;
				int badkeys = 0;
				String links = req.getParameter("links");
				if (! "yes".equals(links))
				{
        				links = "no";
				}
				if ("yes".equals(links))
				{
					res.setContentType("text/html");
				}
				else 
				{
					res.setContentType("text/plain");
				}
				pw = res.getWriter();
				Vector buffer = new Vector();
				String keptKey = null;
				String mode = req.getParameter("mode");
				if ((! "singleton".equals(mode))
				&&  (! "duplicate".equals(mode)) 
				&&  (! "all".equals(mode))) {
        				mode = "singleton";
				}
				if ("yes".equals(links))
				{
					pw.println("<html><head></head><body><pre>");
				}
				while (rs.next ())
				{
					String providerId = rs.getString ("provider_id");
					String siteId = rs.getString ("site_id");
					String key = getKey(providerId);
					//PROTECT AGAINST ABBERANT NULL KEY
					if (key == null || "".equals(key)) {
						badkeys++;
						logger.error("bad key:  [" + providerId + "] => [" + key + "]");
						continue;
					} else {
						logger.debug("good key:  [" + providerId + "] => [" + key + "]");
					}
       					++count;
					String url = publicUrl + siteId;
					if ("yes".equals(links))
					{
						url = "<a href=\"" + url + "\">" + siteId + "</a>";
					}
					String line = key + SEPARATOR + url;
					if ("all".equals(mode)) {
						pw.println(line);
						nOutputLines++;
					} else {
						if (ignore(providerId)) {
							continue;
						}
						if ( ! key.equals(keptKey) ) {
							processBuffer(pw, mode, buffer);
						}
						keptKey = key;
						buffer.add(line);
						nOutputLines++;
					}
   				}
				if (! "all".equals(mode))
				{
					processBuffer(pw, mode, buffer);
				}
				if ("yes".equals(links))
				{
					pw.println("</pre></body></html>");
				}
				logger.info("WDN SYLLABI SERVLET nOutputLines=" + nOutputLines + ", mode=" + mode + ", links=" + links);
			}
		}
           	catch (Exception e)
           	{
			res.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
			logger.fatal("WDN SYLLABI SERVLET problem");
			logger.fatal(e.getMessage());
			if (e.getCause() != null) {
				logger.fatal(e.getCause().getMessage());
			}
           	}
		finally
           	{
			if (pw != null)
			{
				try
				{
					pw.close ();
				}
				catch (Exception e)
				{
					logger.error("SYLLABI LIST SERVLET pw close error");
				}
			}
			if (conn != null)
			{
				try
				{
					conn.close ();
					logger.debug("WDN SYLLABI SERVLET Database connection terminated");
				}
				catch (Exception e)
				{
					logger.error("SYLLABI LIST SERVLET conn close error");
				}
			}
			if (s != null)
			{
				try
				{
					s.close ();
					logger.debug("WDN SYLLABI SERVLET Database connection terminated");
				}
				catch (Exception e)
				{
					logger.error("SYLLABI LIST SERVLET statement close error");
				}
			}
			if (rs != null)
			{
				try
				{
					rs.close ();
					logger.debug("WDN SYLLABI SERVLET Database connection terminated");
				}
				catch (Exception e)
				{
					logger.error("SYLLABI LIST SERVLET result set close error");
				}
			}
           	}
	}

	protected  void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		doGet(req, resp);
	}

} // SyllabiListServlet
